#ifndef _LL_ARCH_X86C_CLOCK_H
#define _LL_ARCH_X86C_CLOCK_H

#include <stdint.h>

// Return a monotonically increasing 64-bit clock value that wraps
// around no more often than once per year, and with a resolution of 1
// us or better (if possible).  This function's availability and
// privileged status is architecture and configuration dependent; on
// x86 and x64, it is normally not privileged.  On some architectures,
// and on older (486 and earlier) x86 chips, it will need to be
// supplied externally using board or kernel specific code.

static inline int64_t ll_getclock()
{
	int32_t high;
	uint32_t low;
	
	asm volatile("rdtsc" : "=a" (low), "=d" (high));

	return ((int64_t)high << 32) | low;
}

static inline int32_t ll_getclock32()
{
	uint32_t clock;
	asm volatile("rdtsc" : "=a" (clock) : : "edx");
	return clock;
}

#endif
