#ifndef _KERN_LIBC_H
#define _KERN_LIBC_H

#include <kern/types.h>
#include <stdarg.h>

size_t vsnprintf(char *buf, size_t size, const char *str, va_list args);
size_t snprintf(char *buf, size_t size, const char *str, ...)
__attribute__((format(printf, 3, 4)));
size_t sprintf(char *buf, const char *str, ...)
__attribute__((format(printf, 2, 3)));
size_t printf(const char *str, ...)
__attribute__((format(printf, 1, 2)));

// These are C-ABI so libgcc and libsupc++ can use them.
extern "C" {
	void *memcpy(void *dest, const void *src, size_t len);
	void *memmove(void *dest, const void *src, size_t len);
	int memcmp(const void *b1, const void *b2, size_t len);
	void *memset(void *b, int ch, size_t len);

	size_t strnlen(const char *s, size_t n);
	size_t strlen(const char *s);

	char *strcpy(char *dest, const char *src);
	char *strncpy(char *dest, const char *src, size_t len);

	void bzero(void *b, size_t len);
	
	void *malloc(size_t size);
	void free(void *ptr);
	void abort();
}

void run_ctors();

// Placement new operators

inline void *operator new(size_t len, void *addr)
{
	return addr;
}

inline void *operator new[](size_t len, void *addr)
{
	return addr;
}

#endif
